@@ -2,7 +2,7 @@
  * plpython.c - python as a procedural language for PostgreSQL
  *
  * This software is copyright by Andrew Bosma
- * but is really shameless cribbed from pltcl.c by Jan Weick, and
+ * but is really shamelessly cribbed from pltcl.c by Jan Wieck, and
  * plperl.c by Mark Hollomon.
  *
  * The author hereby grants permission to use, copy, modify,
@@ -29,7 +29,7 @@
  * MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
  *
  * IDENTIFICATION
- *	$PostgreSQL: pgsql/src/pl/plpython/plpython.c,v 1.58.4.2 2005/09/23 21:02:37 momjian Exp $
+ *	$PostgreSQL: pgsql/src/pl/plpython/plpython.c,v 1.58.4.5 2006/01/17 17:33:37 tgl Exp $
  *
  *********************************************************************
  */
@@ -861,6 +861,7 @@ PLy_function_build_args(FunctionCallInfo fcinfo, PLyProcedure * proc)
 					tupType = HeapTupleHeaderGetTypeId(td);
 					tupTypmod = HeapTupleHeaderGetTypMod(td);
 					tupdesc = lookup_rowtype_tupdesc(tupType, tupTypmod);
+					tupdesc = CreateTupleDescCopy(tupdesc);
 
 					/* Set up I/O funcs if not done yet */
 					if (proc->args[i].is_rowtype != 1)
@@ -871,6 +872,7 @@ PLy_function_build_args(FunctionCallInfo fcinfo, PLyProcedure * proc)
 					tmptup.t_data = td;
 
 					arg = PLyDict_FromTuple(&(proc->args[i]), &tmptup, tupdesc);
+					FreeTupleDesc(tupdesc);
 				}
 			}
 			else
@@ -902,6 +904,7 @@ PLy_function_build_args(FunctionCallInfo fcinfo, PLyProcedure * proc)
 			 * FIXME -- error check this
 			 */
 			PyList_SetItem(args, i, arg);
+			arg = NULL;
 		}
 	}
 	PG_CATCH();
@@ -1998,7 +2001,6 @@ PLy_spi_execute_plan(PyObject * ob, PyObject * list, int limit)
 	int			i,
 				rv;
 	PLyPlanObject *plan;
-	char	   *nulls;
 	MemoryContext oldcontext;
 
 	if (list != NULL)
@@ -2020,8 +2022,8 @@ PLy_spi_execute_plan(PyObject * ob, PyObject * list, int limit)
 	if (nargs != plan->nargs)
 	{
 		char	   *sv;
-
 		PyObject   *so = PyObject_Str(list);
+
 		if (!so)
 			PLy_elog(ERROR, "function \"%s\" could not execute plan",
 					 PLy_procedure_name(PLy_curr_procedure));
@@ -2037,13 +2039,12 @@ PLy_spi_execute_plan(PyObject * ob, PyObject * list, int limit)
 	oldcontext = CurrentMemoryContext;
 	PG_TRY();
 	{
-		nulls = palloc(nargs * sizeof(char));
+		char	   *nulls = palloc(nargs * sizeof(char));
 
 		for (i = 0; i < nargs; i++)
 		{
 			PyObject   *elem,
 					   *so;
-			char	   *sv;
 
 			elem = PySequence_GetItem(list, i);
 			if (elem != Py_None)
@@ -2052,20 +2053,26 @@ PLy_spi_execute_plan(PyObject * ob, PyObject * list, int limit)
 				if (!so)
 					PLy_elog(ERROR, "function \"%s\" could not execute plan",
 							 PLy_procedure_name(PLy_curr_procedure));
-				sv = PyString_AsString(so);
-
-				/*
-				 * FIXME -- if this elogs, we have Python reference leak
-				 */
-				plan->values[i] =
-					FunctionCall3(&(plan->args[i].out.d.typfunc),
-								  CStringGetDatum(sv),
-						ObjectIdGetDatum(plan->args[i].out.d.typioparam),
-								  Int32GetDatum(-1));
-
-				Py_DECREF(so);
 				Py_DECREF(elem);
 
+				PG_TRY();
+				{
+					char *sv = PyString_AsString(so);
+
+					plan->values[i] =
+						FunctionCall3(&(plan->args[i].out.d.typfunc),
+									  CStringGetDatum(sv),
+							ObjectIdGetDatum(plan->args[i].out.d.typioparam),
+									  Int32GetDatum(-1));
+				}
+				PG_CATCH();
+				{
+					Py_DECREF(so);
+					PG_RE_THROW();
+				}
+				PG_END_TRY();
+
+				Py_DECREF(so);
 				nulls[i] = ' ';
 			}
 			else
